Package: lisaClust
Type: Package
Title: lisaClust: Clustering of Local Indicators of Spatial Association
Version: 1.18.0
Authors@R: c(
    person("Ellis", "Patrick", , "ellis.patrick@sydney.edu.au", role = c("aut", "cre")),
    person("Nicolas", "Canete", , "nicolas.canete@sydney.edu.au", role = "aut"),
    person("Nicholas", "Robertson", , "nicholas.robertson@sydney.edu.au", role = "ctb"),
    person("Alex", "Qin", , "alex.qin@sydney.edu.au", role = "ctb"),
    person("Shreya", "Rao", "shreya.rajeshrao@sydney.edu.au", role = "ctb")
  )
Description: lisaClust provides a series of functions to identify and visualise 
    regions of tissue where spatial associations between cell-types is similar.
    This package can be used to provide a high-level summary of cell-type 
    colocalization in multiplexed imaging data that has been segmented at a 
    single-cell resolution.
License: GPL (>=2)
biocViews: SingleCell, CellBasedAssays, Spatial
Encoding: UTF-8
Depends: R (>= 4.1.0)
VignetteBuilder: knitr
BugReports: https://github.com/ellispatrick/lisaClust/issues
URL: https://ellispatrick.github.io/lisaClust/,
        https://github.com/ellispatrick/lisaClust
Imports: ggplot2, class, concaveman, grid, BiocParallel,
        spatstat.explore, spatstat.geom, BiocGenerics, S4Vectors,
        methods, spicyR, purrr, stats, data.table, dplyr, tidyr,
        SingleCellExperiment, SpatialExperiment, SummarizedExperiment,
        pheatmap, spatstat.random, lifecycle, simpleSeg, rlang,
Suggests: SpatialDatasets, BiocStyle, knitr, rmarkdown, testthat (>=
        3.0.0)
RoxygenNote: 7.3.2
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/lisaClust
git_branch: RELEASE_3_22
git_last_commit: 8ecd502
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 04:49:12 UTC; biocbuild
Author: Ellis Patrick [aut, cre],
  Nicolas Canete [aut],
  Nicholas Robertson [ctb],
  Alex Qin [ctb],
  Shreya shreya.rajeshrao@sydney.edu.au Rao [ctb]
Maintainer: Ellis Patrick <ellis.patrick@sydney.edu.au>

# Generated by roxygen2: do not edit by hand

export(geom_hatching)
export(hatchingPlot)
export(inhomLocalK)
export(lisa)
export(lisaClust)
export(regionMap)
export(scale_region)
export(scale_region_manual)
import(SingleCellExperiment)
import(SpatialExperiment)
importFrom(BiocGenerics,do.call)
importFrom(BiocGenerics,rbind)
importFrom(BiocParallel,MulticoreParam)
importFrom(BiocParallel,bplapply)
importFrom(S4Vectors,DataFrame)
importFrom(SpatialExperiment,spatialCoords)
importFrom(SummarizedExperiment,colData)
importFrom(class,knn)
importFrom(concaveman,concaveman)
importFrom(data.table,":=")
importFrom(data.table,.SD)
importFrom(data.table,CJ)
importFrom(data.table,as.data.table)
importFrom(data.table,dcast)
importFrom(data.table,setkey)
importFrom(dplyr,.data)
importFrom(dplyr,bind_rows)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(ggplot2,aes)
importFrom(ggplot2,discrete_scale)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,layer)
importFrom(ggplot2,scale_colour_gradient2)
importFrom(ggplot2,theme_minimal)
importFrom(grid,gList)
importFrom(grid,gpar)
importFrom(grid,grob)
importFrom(grid,grobTree)
importFrom(grid,linesGrob)
importFrom(grid,polylineGrob)
importFrom(lifecycle,deprecate_warn)
importFrom(methods,is)
importFrom(pheatmap,pheatmap)
importFrom(purrr,map)
importFrom(purrr,map_dfr)
importFrom(spatstat.explore,density.ppp)
importFrom(spatstat.explore,localLcross)
importFrom(spatstat.explore,localLcross.inhom)
importFrom(spatstat.geom,area)
importFrom(spatstat.geom,as.mask)
importFrom(spatstat.geom,as.polygonal)
importFrom(spatstat.geom,border)
importFrom(spatstat.geom,closepairs)
importFrom(spatstat.geom,convexhull)
importFrom(spatstat.geom,inside.owin)
importFrom(spatstat.geom,intersect.owin)
importFrom(spatstat.geom,marks)
importFrom(spatstat.geom,nearest.valid.pixel)
importFrom(spatstat.geom,owin)
importFrom(spatstat.geom,ppp)
importFrom(spatstat.geom,solapply)
importFrom(spatstat.geom,union.owin)
importFrom(spatstat.random,expand.owin)
importFrom(stats,kmeans)
importFrom(stats,loess)
importFrom(stats,rpois)
importFrom(stats,var)
importFrom(tidyr,pivot_longer)

# lisaClust

<img src="https://raw.githubusercontent.com/ellispatrick/lisaClust/master/inst/lisaClust_sticker_ai_upscaled.png" align="right" width="200" />

Clustering of Local Indicators of Spatial Association

## Overview


**lisaClust** provides a series of functions to identify and visualise 
    regions of tissue where spatial associations between cell-types is similar.
    This package can be used to provide a high-level summary of cell-type 
    colocalisation in multiplexed imaging data that has been segmented at a 
    single-cell resolution.

## Installation


For the Bioconductor release version, run the following.
```r
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("lisaClust")
```

If you would like the most up-to-date features, install the most recent development version.
```r
# Install the development version from Bioconductor:
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

# The following initializes usage of Bioc devel
# This will update all your Bioconductor packages to devel version
BiocManager::install(version='devel')

BiocManager::install("lisaClust")

# Otherwise install the development version from GitHub:
# install.packages("devtools")
devtools::install_github("ellispatrick/lisaClust")
library(lisaClust)
```
## Submitting an issue or feature request

`listClust` is still under active development. We would greatly appreciate any and 
all feedback related to the package.

* R package related issues should be raised [here](https://github.com/ellispatrick/listClust/issues).
* For general questions and feedback, please contact us directly via [ellis.patrick@sydney.edu.au](mailto:ellis.patrick@sydney.edu.au).


## Authors

* **Nicolas Canete**
* **Ellis Patrick**  - [@TheEllisPatrick](https://twitter.com/TheEllisPatrick)

## Citation

Patrick, E, Canete, NP, Iyengar, SS, Harman, AN, Sutherland, GT, Yang, P. Spatial analysis for highly multiplexed imaging data to identify tissue microenvironments. *Cytometry*. 2023. <https://doi.org/10.1002/cyto.a.24729>

#' Generate local indicators of spatial association
#'
#' @param cells A SingleCellExperiment, SpatialExperiment or data frame that contains at least the
#' variables x and y, giving the  coordinates of each cell, imageID and cellType.
#' @param Rs A vector of the radii that the measures of association should be calculated.
#' @param imageID The column which contains image identifiers.
#' @param cellType The column which contains the cell types.
#' @param spatialCoords The columns which contain the x and y spatial coordinates.
#' @param cores Number of cores to use for parallel processing, or a BiocParallel
#' MulticoreParam or SerialParam object.
#' @param window Should the window around the regions be 'square', 'convex' or 'concave'.
#' @param window.length A tuning parameter for controlling the level of concavity
#' when estimating concave windows.
#' @param whichParallel Should the function use parallization on the imageID or
#' the cellType.
#' @param sigma A numeric variable used for scaling when filting inhomogeneous L-curves.
#' @param lisaFunc Either "K" or "L" curve.
#' @param minLambda  Minimum value for density for scaling when fitting inhomogeneous L-curves.
#' @param BPPARAM \{DEPRECATED\} A BiocParallel MulticoreParam or SerialParam object. 
#'
#' @return A matrix of LISA curves
#'
#' @examples
#' library(spicyR)
#' library(SingleCellExperiment)
#' # Read in data
#' isletFile <- system.file("extdata", "isletCells.txt.gz", package = "spicyR")
#' cells <- read.table(isletFile, header = TRUE)
#' cellExp <- SingleCellExperiment(
#'   assay = list(intensities = t(cells[, grepl(names(cells), pattern = "Intensity_")])),
#'   colData = cells[, !grepl(names(cells), pattern = "Intensity_")]
#' )
#'
#' # Cluster cell types
#' markers <- t(assay(cellExp, "intensities"))
#' kM <- kmeans(markers, 8)
#' colData(cellExp)$cluster <- paste("cluster", kM$cluster, sep = "")
#'
#' # Generate LISA
#' lisaCurves <- lisa(
#'   cellExp,
#'   spatialCoords = c("Location_Center_X", "Location_Center_Y"),
#'   cellType = "cluster", imageID = "ImageNumber"
#' )
#'
#' # Cluster the LISA curves
#' kM <- kmeans(lisaCurves, 2)
#'
#' @export
#' @rdname lisa
#' @importFrom methods is
#' @importFrom BiocParallel MulticoreParam bplapply
#' @importFrom S4Vectors DataFrame
#' @importFrom BiocGenerics do.call rbind
#' @importFrom dplyr bind_rows
lisa <- function(cells,
                 Rs = NULL,
                 imageID = "imageID",
                 cellType = "cellType",
                 spatialCoords = c("x", "y"),
                 cores = 1,
                 window = "convex",
                 window.length = NULL,
                 whichParallel = "imageID",
                 sigma = NULL,
                 lisaFunc = "K",
                 minLambda = 0.05,
                 BPPARAM = BiocParallel::SerialParam()) {
  
  user_args = as.list(match.call())[-1]
  
  tryCatch({
    user_vals = lapply(names(user_args), function(arg) {
      if (arg %in% c("BPPARAM", "cores")) {
        eval(user_args[[arg]])
      } 
    })
    
    names(user_vals) = names(user_args)
    
    if (length(user_vals) > 0 && any(!sapply(user_vals, is.null))) {
      argumentChecks("lisaClust", user_vals)
    }
  }, error = function(e) {
    if (grepl("object 'cd' not found", e$message)) {
      message("Skipping argument checks as `lisa()` is being called within `lisaClust()`")
    } else {
      stop(e)
    }
  })
  
  if (is(cells, "SummarizedExperiment")) {
    cols = colnames(colData(cells))
  } else if (is(cells, "data.frame")) {
    cols = colnames(cells)
  } else {
    stop("Data must be in the form of a SingleCellExperiment, SpatialExperiment, or data frame.")
  }
  
  if (!(imageID %in% cols)) {
    stop(paste0("'", imageID, "' column not found in data"))
  }
  
  if (!(cellType %in% cols)) {
    stop(paste0("'", cellType, "' column not found in data"))
  }
  
  if (methods::is(cells, "SummarizedExperiment")) {
    cells <- spicyR:::.format_data(
      cells, imageID, cellType, spatialCoords, FALSE
    )
  }
  
  cellSummary <- spicyR:::getCellSummary(cells, bind = FALSE)
  
  if (is.null(Rs)) {
    Rs <- c(20, 50, 100)
  }
  
  if (is(cores, "numeric")) {
    cores = BiocParallel::MulticoreParam(workers = cores)
  } 
  
  if (whichParallel == "imageID") {
    BPimage <- cores
  }
  if (whichParallel == "cellType") {
    BPcellType <- cores
  }
  
  
  message("Generating local L-curves.")
  
  curveList <-
    BiocParallel::bplapply(
      cellSummary,
      inhomLocalK,
      Rs = Rs,
      sigma = sigma,
      window = window,
      window.length = window.length,
      minLambda = minLambda,
      lisaFunc = lisaFunc,
      BPPARAM = cores
    )
  
  curvelist <- lapply(curveList, as.data.frame)
  curves <- as.matrix(dplyr::bind_rows(curvelist))
  rownames(curves) <- as.character(unlist(lapply(cellSummary, function(x) x$cellID)))
  
  curves[is.na(curves)] <- 0
  return(curves)
}




#' @importFrom spatstat.geom ppp
pppGenerate <- function(cells, window, window.length) {
  ow <- makeWindow(cells, window, window.length)
  pppCell <- spatstat.geom::ppp(
    cells$x,
    cells$y,
    window = ow,
    marks = cells$cellType
  )
  
  pppCell
}

#' @importFrom spatstat.geom owin convexhull ppp
#' @importFrom concaveman concaveman
makeWindow <-
  function(data,
           window = "square",
           window.length = NULL) {
    data <- data.frame(data)
    ow <-
      spatstat.geom::owin(xrange = range(data$x), yrange = range(data$y))
    
    if (window == "convex") {
      p <- spatstat.geom::ppp(data$x, data$y, ow)
      ow <- spatstat.geom::convexhull(p)
    }
    if (window == "concave") {
      message("Concave windows are temperamental. Try choosing values of window.length > and < 1 if you have problems.")
      if (is.null(window.length)) {
        window.length <- (max(data$x) - min(data$x)) / 20
      } else {
        window.length <- (max(data$x) - min(data$x)) / 20 * window.length
      }
      dist <- (max(data$x) - min(data$x)) / (length(data$x))
      bigDat <-
        do.call("rbind", lapply(as.list(as.data.frame(t(data[, c("x", "y")]))), function(x) {
          cbind(
            x[1] + c(0, 1, 0, -1, -1, 0, 1, -1, 1) * dist,
            x[2] + c(0, 1, 1, 1, -1, -1, -1, 0, 0) * dist
          )
        }))
      ch <-
        concaveman::concaveman(bigDat,
                               length_threshold = window.length,
                               concavity = 1
        )
      poly <- as.data.frame(ch[nrow(ch):1, ])
      colnames(poly) <- c("x", "y")
      ow <-
        spatstat.geom::owin(
          xrange = range(poly$x),
          yrange = range(poly$y),
          poly = poly
        )
    }
    ow
  }


#' @importFrom spatstat.geom union.owin border inside.owin solapply intersect.owin area
borderEdge <- function(X, maxD) {
  W <- X$window
  bW <- spatstat.geom::union.owin(
    spatstat.geom::border(W, maxD, outside = FALSE),
    spatstat.geom::border(W, 2, outside = TRUE)
  )
  inB <- spatstat.geom::inside.owin(X$x, X$y, bW)
  e <- rep(1, X$n)
  if (any(inB)) {
    circs <- spatstat.geom::discs(X[inB], maxD, separate = TRUE)
    circs <- spatstat.geom::solapply(circs, spatstat.geom::intersect.owin, X$window)
    areas <- unlist(lapply(circs, spatstat.geom::area)) / (pi * maxD^2)
    e[inB] <- areas
  }
  
  e
}





#' @importFrom spatstat.geom ppp
#' @importFrom spatstat.explore localLcross localLcross.inhom density.ppp
#' @importFrom BiocParallel bplapply
generateCurves <-
  function(data,
           Rs,
           window,
           window.length,
           BPcellType = BPcellType,
           sigma = sigma,
           ...) {
    ow <- makeWindow(data, window, window.length)
    p1 <-
      spatstat.geom::ppp(
        x = data$x,
        y = data$y,
        window = ow,
        marks = data$cellType
      )
    
    if (!is.null(sigma)) {
      d <- spatstat.explore::density.ppp(p1, sigma = sigma)
      d <- d / mean(d)
    }
    
    
    locIJ <-
      BiocParallel::bplapply(as.list(levels(p1$marks)), function(j) {
        locI <- lapply(as.list(levels(p1$marks)), function(i) {
          iID <- data$cellID[p1$marks == i]
          jID <- data$cellID[p1$marks == j]
          locR <- matrix(NA, length(iID), length(Rs))
          rownames(locR) <- iID
          
          if (length(jID) > 1 & length(iID) > 1) {
            if (!is.null(sigma)) {
              dFrom <- d * (sum(p1$marks == i) - 1) / spatstat.geom::area(ow)
              dTo <-
                d * (sum(p1$marks == j) - 1) / spatstat.geom::area(ow)
              localL <-
                spatstat.explore::localLcross.inhom(
                  p1,
                  from = i,
                  to = j,
                  verbose = FALSE,
                  lambdaFrom = dFrom,
                  lambdaTo = dTo
                )
            } else {
              localL <-
                spatstat.explore::localLcross(
                  p1,
                  from = i,
                  to = j,
                  verbose = FALSE
                )
            }
            ur <-
              vapply(Rs, function(x) {
                which.min(abs(localL$r - x))[1]
              }, numeric(1))
            locR <-
              t(apply(
                as.matrix(localL)[, grep("iso", colnames(localL))],
                2, function(x) {
                  ((x - localL$theo) / localL$theo)[ur]
                }
              ))
            rownames(locR) <- iID
          }
          colnames(locR) <-
            paste(j, round(Rs, 2), sep = "_")
          
          locR
        })
        do.call("rbind", locI)
      }, BPPARAM = BPcellType)
    do.call("cbind", locIJ)
  }

#' @importFrom stats loess rpois var
sqrtVar <- function(x) {
  len <- 1000
  lambda <- (seq(1, 300, length.out = len) / 100)^x
  mL <- max(lambda)
  
  
  V <- NULL
  for (i in 1:len) {
    V[i] <- var(sqrt(rpois(10000, lambda[i])))
  }
  
  lambda <- lambda^(1 / x)
  V <- V
  
  f <- loess(V ~ lambda, span = 0.1)
}



#' @importFrom spatstat.geom nearest.valid.pixel area marks
weightCounts <- function(dt, X, maxD, lam) {
  maxD <- as.numeric(as.character(maxD))
  
  # edge correction
  e <- borderEdge(X, maxD)
  
  # lambda <- as.vector(e%*%t(maxD^2*lam*pi))
  # pred <- predict(fit,lambda^(1/4))
  # pred[lambda < 0.001] = (lambda - 4*lambda^2)[lambda < 0.001]
  # pred[lambda > mL^(1/4)] = 0.25
  # V <- e%*%t(maxD^2*lam*pi)
  # V[] <- pred
  
  
  lambda <- as.vector(maxD^2 * lam * pi)
  names(lambda) <- names(lam)
  LE <- (e) %*% t(lambda)
  mat <- apply(dt, 2, function(x) x)
  mat <- ((mat) - (LE))
  mat <- mat / sqrt(LE)
  
  #   # plot(apply(mat,2,sd))
  #   # plot(apply(mat,2,mean))
  colnames(mat) <- paste(maxD, colnames(mat), sep = "_")
  mat
}

#' Calculate the inhomogenous local K function.
#'
#' @param data The data.
#' @param Rs
#'   A vector of the radii that the measures of association should be
#'   calculated.
#' @param sigma
#'   A numeric variable used for scaling when filting inhomogeneous L-curves.
#' @param window
#'   Should the window around the regions be 'square', 'convex' or 'concave'.
#' @param window.length
#'   A tuning parameter for controlling the level of concavity.
#' @param minLambda
#'   Minimum value for density for scaling when fitting inhomogeneous L-curves.
#' @param lisaFunc Either "K" or "L" curve.
#'
#' @return A matrix of LISA curves
#'
#' @examples
#' library(spicyR)
#' # Read in data
#' isletFile <- system.file("extdata", "isletCells.txt.gz", package = "spicyR")
#' cells <- read.table(isletFile, header = TRUE)
#' cells$x <- cells$AreaShape_Center_X
#' cells$y <- cells$AreaShape_Center_Y
#' cells$cellType <- as.factor(sample(
#'   c("big", "medium", "small"),
#'   length(cells$AreaShape_Center_Y),
#'   replace = TRUE
#' ))
#' cells$cellID <- as.factor(cells$ObjectNumber)
#'
#' inhom <- inhomLocalK(cells[1:100, ])
#'
#' @export
#' @rdname inhomLocalK
#' @importFrom spatstat.geom ppp closepairs marks area
#' @importFrom spatstat.explore density.ppp
#' @importFrom spatstat.random expand.owin
#' @importFrom tidyr pivot_longer
#' @importFrom dplyr left_join
inhomLocalK <-
  function(data,
           Rs = c(20, 50, 100, 200),
           sigma = 10000,
           window = "convex",
           window.length = NULL,
           minLambda = 0.05,
           lisaFunc = "K") {
    ow <- makeWindow(data, window, window.length)
    ow <- spatstat.random::expand.owin(ow, distance = 0.01)
    X <-
      spatstat.geom::ppp(
        x = data$x,
        y = data$y,
        window = ow,
        marks = data$cellType
      )
    
    if (is.null(Rs)) {
      Rs <- c(20, 50, 100, 200)
    }
    if (is.null(sigma)) {
      sigma <- 100000
    }
    
    maxR <- min(ow$xrange[2] - ow$xrange[1], ow$yrange[2] - ow$yrange[1]) / 2.01
    Rs <- unique(pmin(c(0, sort(Rs)), maxR))
    
    den <- spatstat.explore::density.ppp(X, sigma = sigma)
    den <- den / mean(den)
    den$v <- pmax(den$v, minLambda)
    
    p <- spatstat.geom::closepairs(X, max(Rs), what = "ijd")
    n <- X$n
    p$j <- data$cellID[p$j]
    p$i <- data$cellID[p$i]
    
    cT <- data$cellType
    names(cT) <- data$cellID
    
    p$d <- cut(p$d, Rs, labels = Rs[-1], include.lowest = TRUE)
    
    # inhom density
    np <- spatstat.geom::nearest.valid.pixel(X$x, X$y, den)
    w <- den$v[cbind(np$row, np$col)]
    names(w) <- data$cellID
    p$wt <- 1 / w[p$j] * mean(w)
    rm(np)
    
    lam <- table(data$cellType) / spatstat.geom::area(X)
    
    
    
    p$cellTypeJ <- cT[p$j]
    p$cellTypeI <- cT[p$i]
    p$i <- factor(p$i, levels = data$cellID)
    edge <- sapply(Rs[-1], function(x) borderEdge(X, x))
    edge <- as.data.frame(edge)
    colnames(edge) <- Rs[-1]
    edge$i <- data$cellID
    edge <- tidyr::pivot_longer(edge, -i, names_to = "d")
    
    p <- dplyr::left_join(as.data.frame(p), edge, c("i", "d"))
    p$d <- factor(p$d, levels = Rs[-1])
    
    
    
    p <- as.data.frame(p)
    
    if (lisaFunc == "K") {
      r <- getK(p, lam)
    }
    if (lisaFunc == "L") {
      r <- getL(p, lam)
    }
    
    as.matrix(r[data$cellID, ])
  }


#' @importFrom data.table as.data.table setkey CJ dcast .SD ":="
getK <-
  function(p, lam) {
    r <- data.table::as.data.table(p)
    r$wt <- r$wt
    r <- r[, j := NULL]
    r <- r[, cellTypeI := NULL]
    data.table::setkey(r, i, d, cellTypeJ, value)
    r <- r[data.table::CJ(i, d, cellTypeJ, unique = TRUE)][, lapply(.SD, sum), by = .(i, d, cellTypeJ, value)][is.na(wt), wt := 0]
    r <- r[, wt := cumsum(wt), by = list(i, cellTypeJ)]
    r$value[is.na(r$value)] <- 1
    E <- as.numeric(as.character(r$d))^2 * pi * r$value * as.numeric(lam[r$cellTypeJ])
    r$wt <- (r$wt - E) / sqrt(E)
    r <- r[, value := NULL]
    r <- data.table::dcast(r, i ~ d + cellTypeJ, value.var = "wt")
    
    r <- as.data.frame(r)
    rownames(r) <- r$i
    r <- r[, -1]
    
    r
  }


#' @importFrom data.table as.data.table setkey CJ dcast .SD ":="
getL <-
  function(p, lam) {
    r <- data.table::as.data.table(p)
    r$wt <- r$wt
    r <- r[, j := NULL]
    r <- r[, cellTypeI := NULL]
    data.table::setkey(r, i, d, cellTypeJ, value)
    r <- r[data.table::CJ(i, d, cellTypeJ, unique = TRUE)][, lapply(.SD, sum), by = .(i, d, cellTypeJ, value)][is.na(wt), wt := 0]
    r <- r[, wt := cumsum(wt), by = list(i, cellTypeJ)]
    r$value[is.na(r$value)] <- 1
    E <- as.numeric(as.character(r$d))^2 * pi * r$value * as.numeric(lam[r$cellTypeJ])
    r$wt <- sqrt(r$wt) - sqrt(E)
    r <- r[, value := NULL]
    r <- data.table::dcast(r, i ~ d + cellTypeJ, value.var = "wt")
    
    r <- as.data.frame(r)
    rownames(r) <- r$i
    r <- r[, -1]
    
    r
  }




#' Plot heatmap of cell type enrichment for lisaClust regions
#'
#' @param cells SingleCellExperiment, SpatialExperiment or data.frame
#' @param type Make a "bubble" or "heatmap" plot.
#' @param region The column storing the regions
#' @param cellType The column storing the cell types
#' @param limit limits to the lower and upper relative frequencies
#' @param ... Any arguments to be passed to the pheatmap package
#'
#' @return A bubble plot or heatmap
#'
#'
#' @examples
#'
#' set.seed(51773)
#' x <- round(c(
#'   runif(200), runif(200) + 1, runif(200) + 2, runif(200) + 3,
#'   runif(200) + 3, runif(200) + 2, runif(200) + 1, runif(200)
#' ), 4) * 100
#' y <- round(c(
#'   runif(200), runif(200) + 1, runif(200) + 2, runif(200) + 3,
#'   runif(200), runif(200) + 1, runif(200) + 2, runif(200) + 3
#' ), 4) * 100
#' cellType <- factor(paste("c", rep(rep(c(1:2), rep(200, 2)), 4), sep = ""))
#' imageID <- rep(c("s1", "s2"), c(800, 800))
#'
#' cells <- data.frame(x, y, cellType, imageID)
#'
#' cells <- lisaClust(cells, k = 2)
#'
#' regionMap(cells)
#'
#' @export
#' @importFrom SummarizedExperiment colData
#' @importFrom pheatmap pheatmap
#' @importFrom ggplot2 ggplot aes geom_point scale_colour_gradient2 theme_minimal labs
#' @importFrom dplyr mutate
#' @import SpatialExperiment SingleCellExperiment
regionMap <- function(cells, type = "bubble", cellType = "cellType", region = "region", limit = c(0.33, 3), ...) {
  if (is.data.frame(cells)) {
    df <- cells[, c(cellType, region)]
  }
  
  if (is(cells, "SingleCellExperiment") | is(cells, "SpatialExperiment")) {
    df <- as.data.frame(SummarizedExperiment::colData(cells))[, c(cellType, region)]
  }
  
  tab <- table(df[, cellType], df[, region])
  tab <- tab / rowSums(tab) %*% t(colSums(tab)) * sum(tab)
  
  ph <- pheatmap::pheatmap(pmax(pmin(tab, limit[2]), limit[1]), cluster_cols = FALSE, silent = TRUE, ...)
  
  if (type == "bubble") {
    p1 <- tab |>
      as.data.frame() |>
      dplyr::mutate(cellType = factor(Var1, levels = levels(Var1)[ph$tree_row$order]), region = Var2, Freq2 = pmax(pmin(Freq, limit[2]), limit[1])) |>
      ggplot2::ggplot(ggplot2::aes(x = region, y = cellType, colour = Freq2, size = Freq2)) +
      ggplot2::geom_point() +
      ggplot2::scale_colour_gradient2(low = "#4575B4", mid = "grey90", high = "#D73027", midpoint = 1, guide = "legend") +
      ggplot2::theme_minimal() +
      ggplot2::labs(x = "Region", y = "Cell-type", colour = "Relative\nFrequency", size = "Relative\nFrequency")
    
    return(p1)
  }
  
  pheatmap::pheatmap(pmax(pmin(tab, limit[2]), limit[1]), cluster_cols = FALSE, ...)
}

#' hatchingPlot
#'
#' The hatchingPlot() function is used to create hatching patterns for representating
#' spatial regions and cell-types.
#'
#' @param cells A data.frame or SingleCellExperiment.
#' @param useImages A vector of images to plot.
#' @param region The region column to plot.
#' @param imageID The imageIDs column if using data.frame or SingleCellExperiment.
#' @param cellType The cellType column if using data.frame or SingleCellExperiment.
#' @param spatialCoords The spatial coordinates columns if using data.frame or SingleCellExperiment.
#' @param window Should the window around the regions be 'square', 'convex' or 'concave'.
#' @param line.spacing A integer indicating the spacing between hatching lines.
#' @param hatching.colour Colour for the hatching.
#' @param nbp An integer tuning the granularity of the grid used when defining regions.
#' @param window.length A tuning parameter for controlling the level of concavity
#' when estimating concave windows.
#'
#' @return A ggplot object
#'
#' @examples
#' ## Generate toy data
#' set.seed(51773)
#' x <- round(c(
#'     runif(200), runif(200) + 1, runif(200) + 2, runif(200) + 3,
#'     runif(200) + 3, runif(200) + 2, runif(200) + 1, runif(200)
#' ), 4) * 100
#' y <- round(c(
#'     runif(200), runif(200) + 1, runif(200) + 2, runif(200) + 3,
#'     runif(200), runif(200) + 1, runif(200) + 2, runif(200) + 3
#' ), 4) * 100
#' cellType <- factor(paste("c", rep(rep(c(1:2), rep(200, 2)), 4), sep = ""))
#' imageID <- rep(c("s1", "s2"), c(800, 800))
#' cells <- data.frame(x, y, cellType, imageID)
#' cells <- SingleCellExperiment::SingleCellExperiment(colData = cells)
#'
#' ## Generate regions
#' cells <- lisaClust(cells, k = 2)
#'
#' ## Plot regions
#' hatchingPlot(cells)
#'
#' @export
#' @rdname hatchingPlot
#' @importFrom ggplot2 ggplot aes geom_point theme_minimal facet_wrap labs
#' @importFrom dplyr .data
hatchingPlot <-
  function(cells,
           useImages = NULL,
           region = "region",
           imageID = "imageID",
           cellType = "cellType",
           spatialCoords = c("x", "y"),
           window = "concave",
           line.spacing = 21,
           hatching.colour = 1,
           nbp = 50,
           window.length = NULL) {
    df <- spicyR:::getCellSummary(
      spicyR:::.format_data(
        cells, imageID, cellType, spatialCoords, FALSE
      ),
      bind = TRUE
    )
    
    regionCol <- c(colData(cells)[, region])
    
    df <- data.frame(df, 
                     region = regionCol)
    
    if (is.null(useImages)) useImages <- df$imageID[1]
    
    if (any(!useImages %in% df$imageID)) {
      stop("Some of the useImages are not in your data.")
    }
    
    df <- df[df$imageID %in% useImages, ]
    p <-
      ggplot(df, aes(
        x = .data$x,
        y = .data$y,
        colour = cellType
      )) +
      geom_point() +
      facet_wrap(~imageID) +
      geom_hatching(
        aes(region = region),
        show.legend = TRUE,
        window = window,
        line.spacing = line.spacing,
        hatching.colour = hatching.colour,
        nbp = nbp,
        window.length = window.length
      )
    q <- p + theme_minimal() + scale_region() + labs(x = "x", y = "y")
    q
  }


################################################################################
##
## geomHatching
##
################################################################################



#' Hatching geom
#'
#' The hatching geom is used to create hatching patterns for representation of spatial regions.
#'
#' @param mapping Set of aesthetic mappings created by aes() or aes_(). If specified
#' and inherit.aes = TRUE (the default), it is combined with the default mapping
#' at the top level of the plot. You must supply mapping if there is no plot mapping.
#' @param data The data to be displayed in this layer. There are three options:
#'
#' If NULL, the default, the data is inherited from the plot data as specified
#' in the call to ggplot(). A data.frame, or other object, will override the plot
#' data. All objects will be fortified to produce a data frame. See fortify() for
#'  which variables will be created. A function will be called with a single argument,
#'  the plot data. The return value must be a data.frame, and will be used as the
#'  layer data. A function can be created from a formula (e.g. ~ head(.x, 10)).
#' @param stat The statistical transformation to use on the data for this layer as a string.
#' @param position adjustment, either as a string, or the result of a call to a
#' position adjustment function.
#' @param show.legend logical. Should this layer be included in the legends? NA,
#' the default, includes if any aesthetics are mapped. FALSE never includes, and
#' TRUE always includes. It can also be a named logical vector to finely select
#' the aesthetics to display.
#' @param inherit.aes If FALSE, overrides the default aesthetics, rather than
#' combining with them. This is most useful for helper functions that define both
#' data and aesthetics and shouldn't inherit behaviour from the default plot
#' specification, e.g. borders().
#' @param na.rm If FALSE, the default, missing values are removed with a warning.
#' If TRUE, missing values are silently removed.
#' @param line.spacing A integer indicating the spacing between hatching lines.
#' @param hatching.colour A colour for the hatching.
#' @param window Should the window around the regions be 'square', 'convex' or 'concave'.
#' @param window.length A tuning parameter for controlling the level of concavity
#' when estimating concave windows.
#' @param nbp An integer tuning the granularity of the grid used when defining regions
#' @param line.width A numeric controlling the width of the hatching lines
#' @param ... Other arguments passed on to layer(). These are often aesthetics,
#' used to set an aesthetic to a fixed value, like colour = "red" or size = 3.
#' They may also be parameters to the paired geom/stat.
#'
#'
#' @return A ggplot geom
#'
#' @examples
#' ## Generate toy data
#' set.seed(51773)
#' library(ggplot2)
#' x <- round(c(
#'     runif(200), runif(200) + 1, runif(200) + 2, runif(200) + 3,
#'     runif(200) + 3, runif(200) + 2, runif(200) + 1, runif(200)
#' ), 4) * 100
#' y <- round(c(
#'     runif(200), runif(200) + 1, runif(200) + 2, runif(200) + 3,
#'     runif(200), runif(200) + 1, runif(200) + 2, runif(200) + 3
#' ), 4) * 100
#' cellType <- factor(paste("c", rep(rep(c(1:2), rep(200, 2)), 4), sep = ""))
#' imageID <- rep(c("s1", "s2"), c(800, 800))
#' cells <- data.frame(x, y, cellType, imageID)
#' ## Generate regions
#' cells <- lisaClust(cells, k = 2)
#'
#' # Plot the regions with geom_hatching()
#' ggplot(
#'     cells, aes(x = x, y = y, colour = cellType, region = region)
#' ) +
#'     geom_point() +
#'     facet_wrap(~imageID) +
#'     geom_hatching()
#'
#' @export
#' @rdname hatchingPlot
#' @importFrom methods is
#' @importFrom BiocParallel bplapply
#' @importFrom ggplot2 layer
geom_hatching <-
  function(mapping = NULL,
           data = NULL,
           stat = "identity",
           position = "identity",
           na.rm = FALSE,
           show.legend = NA,
           inherit.aes = TRUE,
           line.spacing = 21,
           hatching.colour = 1,
           window = "concave",
           window.length = NULL,
           nbp = 250,
           line.width = 1,
           ...) {
    ggplot2::layer(
      geom = GeomHatching,
      mapping = mapping,
      data = data,
      stat = stat,
      position = position,
      show.legend = show.legend,
      inherit.aes = inherit.aes,
      params = list(
        na.rm = na.rm,
        line.spacing = line.spacing,
        hatching.colour = hatching.colour,
        window = window,
        window.length = window.length,
        nbp = nbp,
        line.width = line.width,
        ...
      )
    )
  }

#' Scale constructor for regions
#'
#' Region scale constructor.
#'
#' @param aesthetics The names of the aesthetics that this scale works with
#' @param ... Arguments passed on to discrete_scale
#' @param guide A function used to create a guide or its name. See guides() for more info.
#'
#' @return a ggplot guide
#'
#' @examples
#'
#' library(spicyR)
#' ## Generate toy data
#' set.seed(51773)
#' x <- round(c(
#'     runif(200), runif(200) + 1, runif(200) + 2, runif(200) + 3,
#'     runif(200) + 3, runif(200) + 2, runif(200) + 1, runif(200)
#' ), 4) * 100
#' y <- round(c(
#'     runif(200), runif(200) + 1, runif(200) + 2, runif(200) + 3,
#'     runif(200), runif(200) + 1, runif(200) + 2, runif(200) + 3
#' ), 4) * 100
#' cellType <- factor(paste("c", rep(rep(c(1:2), rep(200, 2)), 4), sep = ""))
#' imageID <- rep(c("s1", "s2"), c(800, 800))
#' cells <- data.frame(x, y, cellType, imageID)
#' cells <- SingleCellExperiment::SingleCellExperiment(colData = cells)
#'
#' ## Generate regions
#' cells <- lisaClust(cells, k = 2)
#'
#' # Plot the regions with hatchingPlot()
#' hatchingPlot(cells) +
#'     scale_region_manual(
#'         values = c(1, 4), labels = c("Region A", "Region B"),
#'         name = "Regions"
#'     )
#'
#' @export
#' @rdname scale_region
#' @importFrom ggplot2 discrete_scale
scale_region <-
  function(aesthetics = "region",
           ...,
           guide = "legend") {
    discrete_scale(
      "region",
      "region_d",
      palette = function(n) 
        seq_len(n),
      ...
    )
  }

#' @export
#' @param values a set of aesthetic values to map data values to. If this is a
#' named vector, then the values will be matched based on the names. If unnamed,
#' values will be matched in order (usually alphabetical) with the limits of the scale.
#' Any data values that don't match will be given na.value.
#' @rdname scale_region
#' @importFrom ggplot2 discrete_scale
scale_region_manual <- function(..., values) {
  force(values)
  pal <- function(n) {
    if (n > length(values)) {
      stop(
        "Insufficient values in manual scale. ",
        n,
        " needed but only ",
        length(values),
        " provided.",
        call. = FALSE
      )
    }
    if (any(!values %in% seq_len(7))) {
      stop("values must be between 1 and 7")
    }
    
    values
  }
  discrete_scale("region", "manual", pal, ...)
}


ggname <- getFromNamespace("ggname", "ggplot2")

#' @importFrom grid grob polylineGrob gpar
draw_key_region <- function(data, params, size) {
  grobs <- grob()
  hatching.colour <- params$hatching.colour
  
  if (data$region == 1) {
    grobs <-
      polylineGrob(
        x = c(0, 0, 1, 1, 0),
        y = c(0, 1, 1, 0, 0),
        id = c(
          1,
          1, 1, 1, 1
        ),
        gp = gpar(col = hatching.colour, lwd = 1)
      )
  }
  
  if (data$region == 2) {
    grobs <-
      polylineGrob(
        x = c(c(0, 0.5), c(0, 1), c(0.5, 1), c(
          0, 0, 1, 1,
          0
        )),
        y = c(c(0.5, 1), c(0, 1), c(0, 0.5), c(0, 1, 1, 0, 0)),
        id = c(
          1,
          1, 2, 2, 3, 3, 4, 4, 4, 4, 4
        ),
        gp = gpar(col = hatching.colour, lwd = 1)
      )
  }
  if (data$region == 3) {
    grobs <-
      polylineGrob(
        x = c(c(1, 0.5), c(1, 0), c(0.5, 0), c(
          0, 0, 1, 1,
          0
        )),
        y = c(c(0.5, 1), c(0, 1), c(0, 0.5), c(0, 1, 1, 0, 0)),
        id = c(
          1,
          1, 2, 2, 3, 3, 4, 4, 4, 4, 4
        ),
        gp = gpar(col = hatching.colour, lwd = 1)
      )
  }
  
  if (data$region == 4) {
    grobs <-
      polylineGrob(
        x = c(c(0, 1), c(0, 1), c(0, 0, 1, 1, 0)),
        y = c(c(
          0.33,
          0.33
        ), c(0.66, 0.66), c(0, 1, 1, 0, 0)),
        id = c(
          1, 1, 2, 2, 3, 3, 3,
          3, 3
        ),
        gp = gpar(col = hatching.colour, lwd = 1)
      )
  }
  
  if (data$region == 5) {
    grobs <-
      polylineGrob(
        x = c(c(0.33, 0.33), c(0.66, 0.66), c(0, 0, 1, 1, 0)),
        y = c(c(0, 1), c(0, 1), c(0, 1, 1, 0, 0)),
        id = c(
          1, 1, 2, 2, 3, 3, 3,
          3, 3
        ),
        gp = gpar(col = hatching.colour, lwd = 1)
      )
  }
  
  
  if (data$region == 6) {
    grobs <-
      polylineGrob(
        x = c(
          c(1, 0.5),
          c(1, 0),
          c(0.5, 0),
          c(0, 0.5),
          c(
            0,
            1
          ),
          c(0.5, 1),
          c(0, 0, 1, 1, 0)
        ),
        y = c(
          c(0.5, 1),
          c(0, 1),
          c(0, 0.5),
          c(0.5, 1),
          c(0, 1),
          c(0, 0.5),
          c(0, 1, 1, 0, 0)
        ),
        id = c(
          1, 1, 2, 2,
          3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 7, 7, 7
        ),
        gp = gpar(col = hatching.colour, lwd = 1)
      )
  }
  
  if (data$region == 7) {
    grobs <-
      polylineGrob(
        x = c(
          c(0, 1),
          c(0, 1),
          c(0.33, 0.33),
          c(0.66, 0.66),
          c(0, 0, 1, 1, 0)
        ),
        y = c(
          c(0.33, 0.33),
          c(0.66, 0.66),
          c(0, 1),
          c(
            0,
            1
          ),
          c(0, 1, 1, 0, 0)
        ),
        id = c(1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 5, 5, 5),
        gp = gpar(col = hatching.colour, lwd = 1)
      )
  }
  grobs$name <- "region_key"
  grobs
}



hatchingLevels <- function(data, hatching = NULL) {
  if (!is.factor(data$region)) {
    data$region <- factor(data$region)
  }
  regionLevels <- levels(data$region)
  if (!any(hatching %in% seq_len(7)) & !is.null(hatching)) {
    stop("hatching must equal the number of regions and be <= 7.")
  }
  if (all(regionLevels %in% names(hatching))) {
    hatching <- hatching[regionLevels]
  }
  if (is.null(hatching)) {
    hatching <- seq_len(length(regionLevels))
    names(hatching) <- regionLevels
  }
  if (length(hatching) == length(regionLevels)) {
    names(hatching) <- regionLevels
  } else {
    stop("hatching must be the same length as the number of regions.")
  }
  return(hatching)
}




GeomHatching <-
  ggplot2::ggproto(
    "GeomHatching",
    ggplot2::GeomPoint,
    extra_params = c(
      "na.rm",
      "line.spacing", "window", "nbp", "line.width", "hatching.colour"
    ),
    draw_panel = function(data,
                          panel_params,
                          coord,
                          na.rm = FALSE,
                          line.spacing = 21,
                          window = "convex",
                          window.length = NULL,
                          nbp = 250,
                          line.width = 1,
                          hatching.colour = 1) {
      coords <- coord$transform(data, panel_params)
      
      if (is.factor(coords$region)) {
        coords$region <-
          as.numeric(coords$region)
      }
      if (is.character(coords$region)) {
        coords$region <-
          as.numeric(as.factor(coords$region))
      }
      
      ow <-
        makeWindow(coords, window, window.length)
      
      pp <-
        spatstat.geom::ppp(coords$x,
                           coords$y,
                           window = ow,
                           marks = coords$region
        )
      
      
      pp$region <-
        pp$marks
      grob <-
        plotRegions(pp,
                    line.spacing,
                    c(0, 1),
                    c(0, 1),
                    nbp = nbp,
                    line.width = line.width,
                    hatching.colour = hatching.colour
        )
      grob$name <-
        "geom_hatching"
      return(grob)
    },
    draw_key = draw_key_region,
    required_aes = c("x", "y", "region"),
    non_missing_aes = c(
      "x",
      "y", "region"
    ),
    default_aes = ggplot2::aes(
      region = 0,
      size = 0.05,
      angle = 0,
      alpha = 1
    )
  )





################################################################################
##
## Create Hatchings
##
################################################################################



######## Plot the hatchings
#' @importFrom purrr map
#' @importFrom grid linesGrob gpar gList grobTree
plotRegions <-
  function(pp,
           line.spacing = 21,
           xrange = c(0, 1),
           yrange = c(0, 1),
           nbp = 250,
           line.width = 1,
           hatching.colour = 1) {
    rx <- xrange
    ry <- yrange
    width <- (rx[2] - rx[1]) / line.spacing
    
    # Convert to 7 regions
    if (max(pp$region) > 7) {
      warning("Can not plot more than 7 regions. Adding regions greater than 7 to region 1.")
      pp$region[pp$region > 7] <- 1
    }
    
    # Convert ppp to grid
    rG <- regionGrid(pp, nbp)
    
    # Convert grid to polygon
    tree <- purrr::map(as.character(unique(pp$region)), ~ {
      rPoly <- regionPoly(rG, .)
      
      bdrys <- purrr::map(rPoly$bdry, ~ {
        df <- do.call("cbind", .)
        df <- data.frame(rbind(df, df[1, ]))
        g <-
          linesGrob(
            x = df$x / rx[2],
            y = df$y / ry[2],
            gp = gpar(col = hatching.colour, lwd = line.width)
          )
        return(g)
      })
      
      hatchFun <-
        switch(.,
               `1` = hatchNull,
               `2` = hatch45,
               `3` = hatch315,
               `4` = hatch90,
               `5` = hatch180,
               `6` = hatchX,
               `7` = hatchPlus
        )
      return(c(
        bdrys,
        hatchFun(rPoly, width, rx, ry, line.width = line.width, hatching.colour = hatching.colour)
      ))
    })
    
    g <- do.call("gList", (do.call("c", tree)))
    return(grobTree(g))
  }






######## Map predicted regions to a regular grid
#' @importFrom class knn
#' @importFrom grid linesGrob gpar gList
#' @importFrom spatstat.geom as.mask
regionGrid <- function(pp, nbp = 250) {
  ow <- pp$window
  m <- spatstat.geom::as.mask(ow, dimyx = c(nbp, nbp))$m
  x <-
    seq(
      from = ow$xrange[1],
      to = ow$xrange[2],
      length.out = nrow(m)
    )
  y <-
    seq(
      from = ow$yrange[1],
      to = ow$yrange[2],
      length.out = ncol(m)
    )
  grid <- expand.grid(x = x, y = y)
  grid <- data.frame(x = grid[, 1], y = grid[, 2])
  df <- as.data.frame(pp)
  k <- rep(NA, length(m))
  K <-
    knn(
      train = df[, c("x", "y")],
      test = grid[t(m)[seq_len(length(m))], ],
      cl = pp$region,
      k = 1
    )
  k[t(m)[seq_len(length(m))]] <- as.character(K)
  data.frame(grid, regions = k)
}


######## Convert grid of regions into a polygon mask for a particular region.
#' @importFrom spatstat.geom owin as.polygonal
regionPoly <- function(grid, region) {
  rx <- range(grid$x)
  ry <- range(grid$y)
  mat <-
    matrix(
      grid$regions == region,
      nrow = length(unique(grid$x)),
      ncol = length(unique(grid$y)),
      byrow = TRUE
    )
  mat[is.na(mat)] <- FALSE
  ow <- spatstat.geom::owin(
    xrange = rx,
    yrange = ry,
    mask = mat
  )
  return(spatstat.geom::as.polygonal(ow))
}


######## Create line grobs of the hatching
#' @importFrom purrr map map_dfr
#' @importFrom grid linesGrob
hatchingLines <-
  function(rPoly,
           allHatch,
           ordColumn,
           h90 = FALSE,
           xr,
           yr,
           rx,
           ry,
           line.width = 1,
           hatching.colour = 1) {
    purrr::map(seq_len(nrow(allHatch)), ~ {
      if (h90) {
        hatch <-
          rbind(c(xr[1], allHatch[., "from"]), c(xr[2], allHatch[., "to"]))
      } else {
        hatch <-
          rbind(c(allHatch[., "from"], yr[1]), c(allHatch[., "to"], yr[2]))
      }
      
      
      intPoints <- purrr::map_dfr(rPoly$bdry, ~ {
        df <- do.call("cbind", .)
        df <- rbind(df, df[1, ])
        colnames(df) <- c("x", "y")
        
        int <- purrr::map_dfr(seq_len(nrow(df) - 1), ~ {
          x1 <- df[., ]
          x2 <- df[. + 1, ]
          return(data.frame(t(
            line.intersection(x1, x2, hatch[1, ], hatch[2, ], interior.only = TRUE)
          )))
        })
        
        x1 <- df[nrow(df), ]
        x2 <- df[1, ]
        int <-
          rbind(
            int,
            line.intersection(hatch[1, ], hatch[2, ], x1, x2, interior.only = TRUE)
          )
        colnames(int) <- c("x", "y")
        return(int)
      })
      
      intPoints <- unique(round(intPoints, 9))
      intPoints <-
        intPoints[!rowSums(intPoints) %in% c("Inf", NA), ]
      linesH <- NULL
      
      if (length(unlist(intPoints)) > 2) {
        intPoints <- intPoints[order(intPoints[, ordColumn]), ]
        from <-
          sort(rep(seq(1, nrow(
            intPoints
          ) - 1, by = 2), 2))
        pointSplit <- split(intPoints, from)
        
        linesH <- purrr::map(pointSplit, ~ {
          return(linesGrob(
            x = .$x / rx[2],
            y = .$y / ry[2],
            gp = gpar(
              col = hatching.colour,
              lwd = line.width
            )
          ))
        })
      }
      return(linesH)
    })
  }


######## No hatching

hatchNull <- function(rPoly, width, rx, ry, line.width = 1, hatching.colour = 1) {
  NULL
}

######## / hatching


hatch45 <- function(rPoly, width, rx, ry, line.width = 1, hatching.colour = 1) {
  xr <- range(rPoly$x)
  yr <- range(rPoly$y)
  from1 <- seq(
    from = xr[1],
    to = xr[2],
    by = width
  )
  to1 <- seq(
    from = xr[2],
    to = xr[2] + xr[2] - xr[1],
    by = width
  )
  from2 <- seq(
    from = xr[1],
    to = 2 * xr[1] - xr[2],
    by = -width
  )
  to2 <- seq(
    from = xr[2],
    to = xr[1],
    by = -width
  )
  allHatch <- data.frame(from = c(from1, from2), to = c(to1, to2))
  
  lines <-
    hatchingLines(
      rPoly,
      allHatch,
      ordColumn = 1,
      h90 = FALSE,
      xr,
      yr,
      rx,
      ry,
      line.width = line.width,
      hatching.colour = hatching.colour
    )
  
  return(do.call("c", lines))
}

######## \ hatching

hatch315 <- function(rPoly, width, rx, ry, line.width = 1, hatching.colour = 1) {
  xr <- range(rPoly$x)
  yr <- range(rPoly$y)
  from1 <- seq(
    from = xr[2],
    to = xr[1],
    by = -width
  )
  to1 <-
    seq(
      from = xr[1],
      to = xr[1] + xr[1] - xr[2],
      by = -width
    )
  from2 <- seq(
    from = xr[2],
    to = 2 * xr[2] - xr[1],
    by = width
  )
  to2 <- seq(
    from = xr[1],
    to = xr[2],
    by = width
  )
  allHatch <- data.frame(from = c(from1, from2), to = c(to1, to2))
  
  lines <-
    hatchingLines(
      rPoly,
      allHatch,
      ordColumn = 1,
      h90 = FALSE,
      xr,
      yr,
      rx,
      ry,
      line.width = line.width,
      hatching.colour = hatching.colour
    )
  
  return(do.call("c", lines))
}

######## | hatching

hatch180 <- function(rPoly, width, rx, ry, line.width = 1, hatching.colour = 1) {
  xr <- range(rPoly$x)
  yr <- range(rPoly$y)
  from1 <- seq(
    from = xr[1],
    to = xr[2],
    by = width
  )
  to1 <- seq(
    from = xr[1],
    to = xr[2],
    by = width
  )
  allHatch <- data.frame(from = c(from1), to = c(to1))
  
  lines <-
    hatchingLines(
      rPoly,
      allHatch,
      ordColumn = 2,
      h90 = FALSE,
      xr,
      yr,
      rx,
      ry,
      line.width = line.width,
      hatching.colour = hatching.colour
    )
  
  return(do.call("c", lines))
}


######## - hatching

hatch90 <- function(rPoly, width, rx, ry, line.width = 1, hatching.colour = 1) {
  xr <- range(rPoly$x)
  yr <- range(rPoly$y)
  from1 <- seq(
    from = yr[1],
    to = yr[2],
    by = width
  )
  to1 <- seq(
    from = yr[1],
    to = yr[2],
    by = width
  )
  allHatch <- data.frame(from = c(from1), to = c(to1))
  
  lines <-
    hatchingLines(
      rPoly,
      allHatch,
      ordColumn = 1,
      h90 = TRUE,
      xr,
      yr,
      rx,
      ry,
      line.width = line.width,
      hatching.colour = hatching.colour
    )
  
  return(do.call("c", lines))
}

######## x hatching

hatchX <- function(rPoly, width, rx, ry, line.width = 1, hatching.colour = 1) {
  h45 <- hatch45(rPoly, width, rx, ry, line.width = line.width, hatching.colour = hatching.colour)
  h315 <- hatch315(rPoly, width, rx, ry, line.width = line.width, hatching.colour = hatching.colour)
  return(c(h45, h315))
}

######## + hatching

hatchPlus <- function(rPoly, width, rx, ry, line.width = 1, hatching.colour = 1) {
  h90 <- hatch90(rPoly, width, rx, ry, line.width = line.width, hatching.colour = hatching.colour)
  h180 <- hatch180(rPoly, width, rx, ry, line.width = line.width, hatching.colour = hatching.colour)
  return(c(h90, h180))
}



####### Calculate intersection of two lines.

line.intersection <-
  function(P1, P2, P3, P4, interior.only = TRUE) {
    ## Modified from the retistruct package to address edge cases.
    P1 <- round(as.vector(P1), 10)
    P2 <- round(as.vector(P2), 10)
    P3 <- round(as.vector(P3), 10)
    P4 <- round(as.vector(P4), 10)
    dx1 <- P1[1] - P2[1]
    dx2 <- P3[1] - P4[1]
    dy1 <- P1[2] - P2[2]
    dy2 <- P3[2] - P4[2]
    D <- det(rbind(c(dx1, dy1), c(dx2, dy2)))
    if (is.na(D) | D == 0) {
      return(c(Inf, Inf))
    }
    D1 <- det(rbind(P1, P2))
    D2 <- det(rbind(P3, P4))
    X <- round(det(rbind(c(D1, dx1), c(D2, dx2))) / D, 10)
    Y <- round(det(rbind(c(D1, dy1), c(D2, dy2))) / D, 10)
    if (interior.only) {
      lambda1 <- -((X - P1[1]) * dx1 + (Y - P1[2]) * dy1) / (dx1^2 + dy1^2)
      lambda2 <-
        -((X - P3[1]) * dx2 + (Y - P3[2]) * dy2) / (dx2^2 + dy2^2)
      if (!((lambda1 >= 0) &
            (lambda1 <= 1) & (lambda2 >= 0) & (lambda2 <= 1))) {
        return(c(NA, NA))
      }
    }
    return(c(X, Y))
  }

#' Use k-means clustering to cluster local indicators of spatial association. For other clustering use lisa.
#'
#' @param cells A SingleCellExperiment, SpatialExperiment or data frame that contains at least the
#' variables x and y, giving the  coordinates of each cell, imageID and cellType.
#' @param k The number of regions to cluster.
#' @param Rs A vector of the radii that the measures of association should be calculated.
#' @param imageID The column which contains image identifiers.
#' @param cellType The column which contains the cell types.
#' @param spatialCoords The columns which contain the x and y spatial coordinates.
#' @param regionName The output column for the lisaClust regions.
#' @param cores Number of cores to use for parallel processing, or a BiocParallel 
#' MulticoreParam or SerialParam object.
#' @param window Should the window around the regions be 'square', 'convex' or 'concave'.
#' @param window.length A tuning parameter for controlling the level of concavity
#' when estimating concave windows.
#' @param whichParallel Should the function use parallization on the imageID or
#' the cellType.
#' @param sigma A numeric variable used for scaling when filting inhomogeneous L-curves.
#' @param lisaFunc Either "K" or "L" curve.
#' @param minLambda  Minimum value for density for scaling when fitting inhomogeneous L-curves.
#' @param BPPARAM \{DEPRECATED\} A BiocParalell MulticoreParam or SerialParam object.
#'
#' @return A matrix of LISA curves
#'
#' @examples
#' library(spicyR)
#' library(SingleCellExperiment)
#' # Read in data
#' isletFile <- system.file("extdata", "isletCells.txt.gz", package = "spicyR")
#' cells <- read.table(isletFile, header = TRUE)
#' cellExp <- SingleCellExperiment(
#'     assay = list(intensities = t(cells[, grepl(names(cells), pattern = "Intensity_")])),
#'     colData = cells[, !grepl(names(cells), pattern = "Intensity_")]
#' )
#'
#' # Cluster cell types
#' markers <- t(assay(cellExp, "intensities"))
#' kM <- kmeans(markers, 8)
#' colData(cellExp)$cluster <- paste("cluster", kM$cluster, sep = "")
#'
#' # Generate LISA
#' cellExp <- lisaClust(cellExp,
#'     k = 2,
#'     imageID = "ImageNumber",
#'     cellType = "cluster",
#'     spatialCoords = c("Location_Center_X", "Location_Center_Y")
#' )

#'
#' @export
#' @rdname lisaClust
#' @importFrom SummarizedExperiment colData
#' @importFrom SpatialExperiment spatialCoords
#' @importFrom BiocParallel MulticoreParam
#' @importFrom stats kmeans
lisaClust <-
  function(cells,
           k = 2,
           Rs = NULL,
           imageID = "imageID",
           cellType = "cellType",
           spatialCoords = c("x", "y"),
           regionName = "region",
           cores = 1,
           window = "convex",
           window.length = NULL,
           whichParallel = "imimageID",
           sigma = NULL,
           lisaFunc = "K",
           minLambda = 0.05,
           BPPARAM = BiocParallel::SerialParam()) {
    
    user_args = as.list(match.call())[-1]
    
    user_vals = lapply(names(user_args), function(arg) {
      if (arg %in% c("BPPARAM", "cores")) {
        eval(user_args[[arg]])
      } 
    })
    
    names(user_vals) = names(user_args)
    argumentChecks("lisaClust", user_vals)
  
    if (is(cells, "SummarizedExperiment")) {
      cols = colnames(colData(cells))
    } else if (is(cells, "data.frame")) {
      cols = colnames(cells)
    } else {
      stop("Data must be in the form of a SingleCellExperiment, SpatialExperiment, or data frame.")
    }
    
    if (!(imageID %in% cols)) {
      stop(paste0("'", imageID, "' column not found in data"))
    }
    
    if (!(cellType %in% cols)) {
      stop(paste0("'", cellType, "' column not found in data"))
    }
    
    if (methods::is(cells, "SummarizedExperiment")) {
      cd <- spicyR:::.format_data(
        cells, imageID, cellType, spatialCoords, FALSE
      )
      
      lisaCurves <- lisa(cd,
                         Rs = Rs,
                         cores = cores,
                         window = window,
                         window.length = window.length,
                         whichParallel = whichParallel,
                         sigma = sigma,
                         lisaFunc = lisaFunc,
                         minLambda = minLambda
      )
      kM <- kmeans(lisaCurves, k)
      regions <- paste("region", kM$cluster, sep = "_")
      
      SummarizedExperiment::colData(cells)[regionName] <- regions
    } else if (is(cells, "data.frame")) {
      cd <- cells
      cd <- cd[, c(cellType, imageID, spatialCoords)]
      colnames(cd) <- c("cellType", "imageID", "x", "y")
      cd$cellID <- as.character(seq_len(nrow(cd)))
      cd$imageCellID <- as.character(seq_len(nrow(cd)))
      
      lisaCurves <- lisa(cd,
                         Rs = Rs,
                         cores = cores,
                         window = window,
                         window.length = window.length,
                         whichParallel = whichParallel,
                         sigma = sigma,
                         lisaFunc = lisaFunc,
                         minLambda = minLambda
      )
      
      kM <- kmeans(lisaCurves, k)
      regions <- paste("region", kM$cluster, sep = "_")
      
      cells[regionName] <- regions
    } else {
      stop(
        "Unsupported datatype for cells: please use",
        "SingleCellExperiment or SpatialExperiment"
      )
    }
    
    cells
  }

test_that(
  "the output is equal to original result", 
  {
    load(system.file("testdata/original_result.rda", package = "lisaClust"))
    set.seed(51773)
    
    kerenSPE <- SpatialDatasets::spe_Keren_2018()
    kerenSPE <- kerenSPE[,colData(kerenSPE)$imageID %in% c("5", "6")]
    kerenSPE <- suppressWarnings(lisaClust(kerenSPE, k = 5)) |>
      colData()
    
    expect_equal(
      suppressWarnings(kerenSPE),
      original_result
    )
  }
)

#' A function to handle validity of argumemts/check for deprecated arguments
#'
#' @importFrom lifecycle deprecate_warn
#' @importFrom methods is
#' @noRd
argumentChecks = function(function_name, user_vals) {
  
  rlang::local_options(lifecycle_verbosity = "warning")  
  
  # handle deprecated arguments
  handle_deprecated = function(old_arg, new_arg, user_vals) {
    if (old_arg %in% names(user_vals) && !(new_arg %in% names(user_vals))) {
      # warning(paste0("'", old_arg, "' was deprecated in 1.18.0. Please use '", new_arg, "' instead.\n"))
      deprecate_warn("1.14.4", paste0(function_name, "(", old_arg, ")"), paste0(function_name, "(", new_arg, ")"))
      assign(new_arg, user_vals[[old_arg]], envir = sys.frame(sys.parent(1)))
    }
  }

  handle_deprecated("BPPARAM", "cores", user_vals)
  
  # enforce mutually exclusive arguments
  check_exclusive = function(arg_set, user_vals) {
    provided_args = intersect(arg_set, names(user_vals)) 
    if (length(provided_args) > 1) {
      stop(paste("Please specify only one of", paste(shQuote(arg_set), collapse = ", "), "\n"))
    }
  }
  
  check_exclusive(c("cores", "BPPARAM"), user_vals)
  
  # validity checks for cores/BPPARAM
  if ("BPPARAM" %in% names(user_vals)) {
    # warning("'BPPARAM' was deprecated in 1.18.0. Please use 'cores' instead.\n")
    deprecate_warn("1.14.4", paste0(function_name, "(BPPARAM)"), paste0(function_name, "(cores)"))
    
    if (is(user_vals$BPPARAM, "MulticoreParam") || is(user_vals$BPPARAM, "SerialParam")) {
      assign("cores", user_vals$BPPARAM, envir = sys.frame(sys.parent(1)))
    } else {
      stop("'BBPARAM' must be a MulticoreParam or SerialParam object.")
    }
    
  } else if ("cores" %in% names(user_vals)) {
    if (is(user_vals$cores, "numeric")) {
      assign("cores", simpleSeg:::generateBPParam(cores = user_vals$cores))
    } else if (is(user_vals$cores, "MulticoreParam") || is(user_vals$cores, "SerialParam")) {
      assign("cores", user_vals$cores, sys.frame(sys.parent(1)))
    } else {
      stop("'cores'  must be either a numeric value, or a MulticoreParam or SerialParam object.\n")
    }
  }
}

---
title: "Introduction to Clustering of Local Indicators of Spatial Assocation (LISA) curves"
date: "`r BiocStyle::doc_date()`"
author:
- name: Nicolas Canete
  affiliation:  
  - &WIMR Westmead Institute for Medical Research, University of Sydney, Australia
  email: nicolas.canete@sydney.edu.au
- name: Ellis Patrick
  affiliation:
  - &WIMR Westmead Institute for Medical Research, University of Sydney, Australia
  - School of Mathematics and Statistics, University of Sydney, Australia
  email: ellis.patrick@sydney.edu.au
- name: Alex Run Qin
  affiliation:
  - &WIMR Westmead Institute for Medical Research, University of Sydney, Australia
  - School of Mathematics and Statistics, University of Sydney, Australia
  email: alex.qin@sydney.edu.au
package: "`r BiocStyle::pkg_ver('lisaClust')`"
abstract: > 
  Identify and visualise regions of cell type colocalization in multiplexed imaging data that has been segmented at a single-cell resolution.
vignette: >
  %\VignetteIndexEntry{"Inroduction to lisaClust"}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
output: 
  BiocStyle::html_document
bibliography: REFERENCES.bib
editor_options: 
  markdown: 
    wrap: 72
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE, message = FALSE, warning = FALSE
)
library(BiocStyle)
```

# Installation

```{r, eval = FALSE}
if (!require("BiocManager")) {
  install.packages("BiocManager")
}
BiocManager::install("lisaClust")
```

```{r message=FALSE, warning=FALSE}
# load required packages
library(lisaClust)
library(spicyR)
library(ggplot2)
library(SingleCellExperiment)
library(SpatialDatasets)
```

# Overview

Clustering local indicators of spatial association (LISA) functions is a
methodology for identifying consistent spatial organisation of multiple
cell-types in an unsupervised way. This can be used to enable the
characterization of interactions between multiple cell-types
simultaneously and can complement traditional pairwise analysis. In our
implementation our LISA curves are a localised summary of an L-function
from a Poisson point process model. Our framework `lisaClust` can be
used to provide a high-level summary of cell-type colocalization in
high-parameter spatial cytometry data, facilitating the identification
of distinct tissue compartments or identification of complex cellular
microenvironments.

# Quick start

## Generate toy data

To illustrate our `lisaClust` framework, we consider a very simple
toy example where two cell-types are completely separated spatially. We
simulate data for two different images.

```{r eval=T}
set.seed(51773)
x <- round(c(
  runif(200), runif(200) + 1, runif(200) + 2, runif(200) + 3,
  runif(200) + 3, runif(200) + 2, runif(200) + 1, runif(200)
), 4) * 100
y <- round(c(
  runif(200), runif(200) + 1, runif(200) + 2, runif(200) + 3,
  runif(200), runif(200) + 1, runif(200) + 2, runif(200) + 3
), 4) * 100
cellType <- factor(paste("c", rep(rep(c(1:2), rep(200, 2)), 4), sep = ""))
imageID <- rep(c("s1", "s2"), c(800, 800))

cells <- data.frame(x, y, cellType, imageID)

ggplot(cells, aes(x, y, colour = cellType)) +
  geom_point() +
  facet_wrap(~imageID) +
  theme_minimal()
```

## Create Single Cell Experiment object

First we store our data in a `SingleCellExperiment` object.

```{r}
SCE <- SingleCellExperiment(colData = cells)
SCE
```

## Running lisaCLust

We can then use the convenience function `lisaClust` to simultaneously
calculate local indicators of spatial association (LISA) functions and perform 
k-means clustering. The number of clusters can be specified with the `k =` parameter. 
In the example below, we've chosen `k = 2`, resulting in a total of 2 clusters. The cell type column can be specified using the `cellType = ` argument. By default, `lisaClust` uses the column named `cellType`.

The clusters identified by `lisaClust` are stored in `colData` of the `SingleCellExperiment`
object as a new column called `regions`.

```{r}
SCE <- lisaClust(SCE, k = 2)
colData(SCE) |> head()
```

## Plot identified regions

`lisaClust` also provides the convenient `hatchingPlot` function to
visualise the different regions that have been demarcated by the
clustering. `hatchingPlot` outputs a `ggplot` object where the regions
are marked by different hatching patterns. In a real biological dataset,
this allows us to plot both regions and cell-types on the same
visualization.

In the example below, we can visualise our stimulated data where our 2
cell types have been separated neatly into 2 distinct regions based on
which cell type each region is dominated by. `region_2` is dominated by
the red cell type `c1`, and `region_1` is dominated by the blue cell
type `c2`.

```{r}
hatchingPlot(SCE, useImages = c("s1", "s2"))
```
## Using other clustering methods.

While the `lisaClust` function is convenient, we have not implemented an exhaustive
suite of clustering methods as it is very easy to do this yourself. There are 
just two simple steps.

### Generate LISA curves

We can calculate local indicators of spatial association (LISA) functions 
using the `lisa` function. Here the LISA curves are a 
localised summary of an L-function from a Poisson point process model. The radii 
that will be calculated over can be set with `Rs`.

```{r}
lisaCurves <- lisa(SCE, Rs = c(20, 50, 100))

head(lisaCurves)
```

### Perform some clustering

The LISA curves can then be used to cluster the cells. Here we use k-means 
clustering. However, other clustering methods like SOM could also be used. We can store these 
cell clusters or cell "regions" in our `SingleCellExperiment` object.

```{r}
# Custom clustering algorithm
kM <- kmeans(lisaCurves, 2)

# Storing clusters into colData
colData(SCE)$custom_region <- paste("region", kM$cluster, sep = "_")
colData(SCE) |> head()
```


# Keren et al. breast cancer data.

Next, we apply our `lisaClust` framework to two images of breast cancer obtained by @keren2018.

## Read in data

We will start by reading in the data from the `SpatialDatasets` package
as a `SingleCellExperiment` object. Here the data is in a format consistent with
that outputted by CellProfiler.

```{r}
kerenSPE <- SpatialDatasets::spe_Keren_2018()
```

## Generate LISA curves

This data includes annotation of the cell-types of each cell. Hence, we can
move directly to performing k-means clustering on the local indicators of
spatial association (LISA) functions using the `lisaClust` function, remembering
to specify the `imageID`, `cellType`, and `spatialCoords` 
columns in `colData`. For the purpose of demonstration, we will be using
only images 5 and 6 of the `kerenSPE` dataset.

```{r}
kerenSPE <- kerenSPE[,kerenSPE$imageID %in% c("5", "6")]

kerenSPE <- lisaClust(kerenSPE,
  k = 5
)
```

These regions are stored in `colData` and can be extracted.

```{r}
colData(kerenSPE)[, c("imageID", "region")] |>
  head(20)
```

## Examine cell type enrichment

`lisaClust` also provides a convenient function, `regionMap`, for examining which 
cell types are located in which regions. In this example, we use this to check
which cell types appear more frequently in each region than expected by chance.

Here, we clearly see that healthy epithelial and mesenchymal tissue are highly
concentrated in region 1, immune cells are concentrated in regions 2 and 4, 
whilst tumour cells are concentrated in region 3.

We can further segregate these cells by increasing the number of clusters, i.e.,
increasing the parameter `k = ` in the `lisaClust()` function. For the purposes
of demonstration, let's take a look at the `hatchingPlot` of these regions.

```{r}
regionMap(kerenSPE,
  type = "bubble"
)
```

## Plot identified regions

Finally, we can use `hatchingPlot` to construct a `ggplot` object where
the regions are marked by different hatching patterns. This allows us to
visualize the 5 regions and 17 cell-types simultaneously.

```{r fig.height=7, fig.width=9}
hatchingPlot(kerenSPE, nbp = 300)
```

# References


# sessionInfo()

```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lisaClust.R
\name{lisaClust}
\alias{lisaClust}
\title{Use k-means clustering to cluster local indicators of spatial association. For other clustering use lisa.}
\usage{
lisaClust(
  cells,
  k = 2,
  Rs = NULL,
  imageID = "imageID",
  cellType = "cellType",
  spatialCoords = c("x", "y"),
  regionName = "region",
  cores = 1,
  window = "convex",
  window.length = NULL,
  whichParallel = "imimageID",
  sigma = NULL,
  lisaFunc = "K",
  minLambda = 0.05,
  BPPARAM = BiocParallel::SerialParam()
)
}
\arguments{
\item{cells}{A SingleCellExperiment, SpatialExperiment or data frame that contains at least the
variables x and y, giving the  coordinates of each cell, imageID and cellType.}

\item{k}{The number of regions to cluster.}

\item{Rs}{A vector of the radii that the measures of association should be calculated.}

\item{imageID}{The column which contains image identifiers.}

\item{cellType}{The column which contains the cell types.}

\item{spatialCoords}{The columns which contain the x and y spatial coordinates.}

\item{regionName}{The output column for the lisaClust regions.}

\item{cores}{Number of cores to use for parallel processing, or a BiocParallel 
MulticoreParam or SerialParam object.}

\item{window}{Should the window around the regions be 'square', 'convex' or 'concave'.}

\item{window.length}{A tuning parameter for controlling the level of concavity
when estimating concave windows.}

\item{whichParallel}{Should the function use parallization on the imageID or
the cellType.}

\item{sigma}{A numeric variable used for scaling when filting inhomogeneous L-curves.}

\item{lisaFunc}{Either "K" or "L" curve.}

\item{minLambda}{Minimum value for density for scaling when fitting inhomogeneous L-curves.}

\item{BPPARAM}{\{DEPRECATED\} A BiocParalell MulticoreParam or SerialParam object.}
}
\value{
A matrix of LISA curves
}
\description{
Use k-means clustering to cluster local indicators of spatial association. For other clustering use lisa.
}
\examples{
library(spicyR)
library(SingleCellExperiment)
# Read in data
isletFile <- system.file("extdata", "isletCells.txt.gz", package = "spicyR")
cells <- read.table(isletFile, header = TRUE)
cellExp <- SingleCellExperiment(
    assay = list(intensities = t(cells[, grepl(names(cells), pattern = "Intensity_")])),
    colData = cells[, !grepl(names(cells), pattern = "Intensity_")]
)

# Cluster cell types
markers <- t(assay(cellExp, "intensities"))
kM <- kmeans(markers, 8)
colData(cellExp)$cluster <- paste("cluster", kM$cluster, sep = "")

# Generate LISA
cellExp <- lisaClust(cellExp,
    k = 2,
    imageID = "ImageNumber",
    cellType = "cluster",
    spatialCoords = c("Location_Center_X", "Location_Center_Y")
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LISA.R
\name{inhomLocalK}
\alias{inhomLocalK}
\title{Calculate the inhomogenous local K function.}
\usage{
inhomLocalK(
  data,
  Rs = c(20, 50, 100, 200),
  sigma = 10000,
  window = "convex",
  window.length = NULL,
  minLambda = 0.05,
  lisaFunc = "K"
)
}
\arguments{
\item{data}{The data.}

\item{Rs}{A vector of the radii that the measures of association should be
calculated.}

\item{sigma}{A numeric variable used for scaling when filting inhomogeneous L-curves.}

\item{window}{Should the window around the regions be 'square', 'convex' or 'concave'.}

\item{window.length}{A tuning parameter for controlling the level of concavity.}

\item{minLambda}{Minimum value for density for scaling when fitting inhomogeneous L-curves.}

\item{lisaFunc}{Either "K" or "L" curve.}
}
\value{
A matrix of LISA curves
}
\description{
Calculate the inhomogenous local K function.
}
\examples{
library(spicyR)
# Read in data
isletFile <- system.file("extdata", "isletCells.txt.gz", package = "spicyR")
cells <- read.table(isletFile, header = TRUE)
cells$x <- cells$AreaShape_Center_X
cells$y <- cells$AreaShape_Center_Y
cells$cellType <- as.factor(sample(
  c("big", "medium", "small"),
  length(cells$AreaShape_Center_Y),
  replace = TRUE
))
cells$cellID <- as.factor(cells$ObjectNumber)

inhom <- inhomLocalK(cells[1:100, ])

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hatchingPlot.R
\name{hatchingPlot}
\alias{hatchingPlot}
\alias{geom_hatching}
\title{hatchingPlot}
\usage{
hatchingPlot(
  cells,
  useImages = NULL,
  region = "region",
  imageID = "imageID",
  cellType = "cellType",
  spatialCoords = c("x", "y"),
  window = "concave",
  line.spacing = 21,
  hatching.colour = 1,
  nbp = 50,
  window.length = NULL
)

geom_hatching(
  mapping = NULL,
  data = NULL,
  stat = "identity",
  position = "identity",
  na.rm = FALSE,
  show.legend = NA,
  inherit.aes = TRUE,
  line.spacing = 21,
  hatching.colour = 1,
  window = "concave",
  window.length = NULL,
  nbp = 250,
  line.width = 1,
  ...
)
}
\arguments{
\item{cells}{A data.frame or SingleCellExperiment.}

\item{useImages}{A vector of images to plot.}

\item{region}{The region column to plot.}

\item{imageID}{The imageIDs column if using data.frame or SingleCellExperiment.}

\item{cellType}{The cellType column if using data.frame or SingleCellExperiment.}

\item{spatialCoords}{The spatial coordinates columns if using data.frame or SingleCellExperiment.}

\item{window}{Should the window around the regions be 'square', 'convex' or 'concave'.}

\item{line.spacing}{A integer indicating the spacing between hatching lines.}

\item{hatching.colour}{A colour for the hatching.}

\item{nbp}{An integer tuning the granularity of the grid used when defining regions}

\item{window.length}{A tuning parameter for controlling the level of concavity
when estimating concave windows.}

\item{mapping}{Set of aesthetic mappings created by aes() or aes_(). If specified
and inherit.aes = TRUE (the default), it is combined with the default mapping
at the top level of the plot. You must supply mapping if there is no plot mapping.}

\item{data}{The data to be displayed in this layer. There are three options:

If NULL, the default, the data is inherited from the plot data as specified
in the call to ggplot(). A data.frame, or other object, will override the plot
data. All objects will be fortified to produce a data frame. See fortify() for
 which variables will be created. A function will be called with a single argument,
 the plot data. The return value must be a data.frame, and will be used as the
 layer data. A function can be created from a formula (e.g. ~ head(.x, 10)).}

\item{stat}{The statistical transformation to use on the data for this layer as a string.}

\item{position}{adjustment, either as a string, or the result of a call to a
position adjustment function.}

\item{na.rm}{If FALSE, the default, missing values are removed with a warning.
If TRUE, missing values are silently removed.}

\item{show.legend}{logical. Should this layer be included in the legends? NA,
the default, includes if any aesthetics are mapped. FALSE never includes, and
TRUE always includes. It can also be a named logical vector to finely select
the aesthetics to display.}

\item{inherit.aes}{If FALSE, overrides the default aesthetics, rather than
combining with them. This is most useful for helper functions that define both
data and aesthetics and shouldn't inherit behaviour from the default plot
specification, e.g. borders().}

\item{line.width}{A numeric controlling the width of the hatching lines}

\item{...}{Other arguments passed on to layer(). These are often aesthetics,
used to set an aesthetic to a fixed value, like colour = "red" or size = 3.
They may also be parameters to the paired geom/stat.}
}
\value{
A ggplot object

A ggplot geom
}
\description{
The hatchingPlot() function is used to create hatching patterns for representating
spatial regions and cell-types.

The hatching geom is used to create hatching patterns for representation of spatial regions.
}
\examples{
## Generate toy data
set.seed(51773)
x <- round(c(
    runif(200), runif(200) + 1, runif(200) + 2, runif(200) + 3,
    runif(200) + 3, runif(200) + 2, runif(200) + 1, runif(200)
), 4) * 100
y <- round(c(
    runif(200), runif(200) + 1, runif(200) + 2, runif(200) + 3,
    runif(200), runif(200) + 1, runif(200) + 2, runif(200) + 3
), 4) * 100
cellType <- factor(paste("c", rep(rep(c(1:2), rep(200, 2)), 4), sep = ""))
imageID <- rep(c("s1", "s2"), c(800, 800))
cells <- data.frame(x, y, cellType, imageID)
cells <- SingleCellExperiment::SingleCellExperiment(colData = cells)

## Generate regions
cells <- lisaClust(cells, k = 2)

## Plot regions
hatchingPlot(cells)

## Generate toy data
set.seed(51773)
library(ggplot2)
x <- round(c(
    runif(200), runif(200) + 1, runif(200) + 2, runif(200) + 3,
    runif(200) + 3, runif(200) + 2, runif(200) + 1, runif(200)
), 4) * 100
y <- round(c(
    runif(200), runif(200) + 1, runif(200) + 2, runif(200) + 3,
    runif(200), runif(200) + 1, runif(200) + 2, runif(200) + 3
), 4) * 100
cellType <- factor(paste("c", rep(rep(c(1:2), rep(200, 2)), 4), sep = ""))
imageID <- rep(c("s1", "s2"), c(800, 800))
cells <- data.frame(x, y, cellType, imageID)
## Generate regions
cells <- lisaClust(cells, k = 2)

# Plot the regions with geom_hatching()
ggplot(
    cells, aes(x = x, y = y, colour = cellType, region = region)
) +
    geom_point() +
    facet_wrap(~imageID) +
    geom_hatching()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hatchingPlot.R
\name{scale_region}
\alias{scale_region}
\alias{scale_region_manual}
\title{Scale constructor for regions}
\usage{
scale_region(aesthetics = "region", ..., guide = "legend")

scale_region_manual(..., values)
}
\arguments{
\item{aesthetics}{The names of the aesthetics that this scale works with}

\item{...}{Arguments passed on to discrete_scale}

\item{guide}{A function used to create a guide or its name. See guides() for more info.}

\item{values}{a set of aesthetic values to map data values to. If this is a
named vector, then the values will be matched based on the names. If unnamed,
values will be matched in order (usually alphabetical) with the limits of the scale.
Any data values that don't match will be given na.value.}
}
\value{
a ggplot guide
}
\description{
Region scale constructor.
}
\examples{

library(spicyR)
## Generate toy data
set.seed(51773)
x <- round(c(
    runif(200), runif(200) + 1, runif(200) + 2, runif(200) + 3,
    runif(200) + 3, runif(200) + 2, runif(200) + 1, runif(200)
), 4) * 100
y <- round(c(
    runif(200), runif(200) + 1, runif(200) + 2, runif(200) + 3,
    runif(200), runif(200) + 1, runif(200) + 2, runif(200) + 3
), 4) * 100
cellType <- factor(paste("c", rep(rep(c(1:2), rep(200, 2)), 4), sep = ""))
imageID <- rep(c("s1", "s2"), c(800, 800))
cells <- data.frame(x, y, cellType, imageID)
cells <- SingleCellExperiment::SingleCellExperiment(colData = cells)

## Generate regions
cells <- lisaClust(cells, k = 2)

# Plot the regions with hatchingPlot()
hatchingPlot(cells) +
    scale_region_manual(
        values = c(1, 4), labels = c("Region A", "Region B"),
        name = "Regions"
    )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LISA.R
\name{regionMap}
\alias{regionMap}
\title{Plot heatmap of cell type enrichment for lisaClust regions}
\usage{
regionMap(
  cells,
  type = "bubble",
  cellType = "cellType",
  region = "region",
  limit = c(0.33, 3),
  ...
)
}
\arguments{
\item{cells}{SingleCellExperiment, SpatialExperiment or data.frame}

\item{type}{Make a "bubble" or "heatmap" plot.}

\item{cellType}{The column storing the cell types}

\item{region}{The column storing the regions}

\item{limit}{limits to the lower and upper relative frequencies}

\item{...}{Any arguments to be passed to the pheatmap package}
}
\value{
A bubble plot or heatmap
}
\description{
Plot heatmap of cell type enrichment for lisaClust regions
}
\examples{

set.seed(51773)
x <- round(c(
  runif(200), runif(200) + 1, runif(200) + 2, runif(200) + 3,
  runif(200) + 3, runif(200) + 2, runif(200) + 1, runif(200)
), 4) * 100
y <- round(c(
  runif(200), runif(200) + 1, runif(200) + 2, runif(200) + 3,
  runif(200), runif(200) + 1, runif(200) + 2, runif(200) + 3
), 4) * 100
cellType <- factor(paste("c", rep(rep(c(1:2), rep(200, 2)), 4), sep = ""))
imageID <- rep(c("s1", "s2"), c(800, 800))

cells <- data.frame(x, y, cellType, imageID)

cells <- lisaClust(cells, k = 2)

regionMap(cells)

}
